/*
 *	SCCS: @(#)systab.c	1.3 (00/04/03)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1996 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 * A copy of the end-user licence agreement is contained in the file
 * Licence which accompanies this distribution.
 * 
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
static char sccsid[] = "@(#)systab.c	1.3 (00/04/03) TET3 release 3.6";
#endif

/************************************************************************

SCCS:   	@(#)systab.c	1.3 00/04/03 TETware release 3.6
NAME:		systab.c
PRODUCT:	TETware
AUTHOR:		Andrew Dingwall, UniSoft Ltd.
DATE CREATED:	August 1996

DESCRIPTION:
	system table administration functions

MODIFICATIONS:
	Andrew Dingwall, UniSoft Ltd., March 2000
	Corrected a problem in ist2() which would sometimes miss out
	system IDs.

************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include <errno.h>
#include "dtmac.h"
#include "dtmsg.h"
#include "ptab.h"
#include "bstring.h"
#include "error.h"
#include "llist.h"
#include "config.h"
#include "scentab.h"
#include "dirtab.h"
#include "systab.h"
#include "tcc.h"

#ifndef NOTRACE
#include "ltoa.h"
#endif

#ifdef NEEDsrcFile
static char srcFile[] = __FILE__;	/* file name for error reporting */
#endif

static struct systab *systab;		/* head of the system table */

/* static function declarations */
static void syadd PROTOLIST((struct systab *));
static struct systab *syalloc PROTOLIST((void));
#if 0
static void syfree PROTOLIST((struct systab *));
static void syrm PROTOLIST((struct systab *));
#endif


/*
**	initsystab() - allocate a tcc system table element for each
**		system mentioned in the scenario tree
*/

void initsystab()
{
	register struct systab *sp;

	TRACE1(tet_Ttcc, 2, "initsystab(): set up a systab for each system mentioned in the chosen scenario");


	sp = syalloc();
	sp->sy_sysid = 0;
	syadd(sp);


	TRACE1(tet_Ttcc, 2, "initsystab() RETURN");
}




/*
**	syalloc(), syfree() - functions to allocate and free a
**		tcc system table element
*/

static struct systab *syalloc()
{
	register struct systab *sp;

	errno = 0;
	if ((sp = (struct systab *) malloc(sizeof *sp)) == (struct systab *) 0)
		fatal(errno, "can't allocate system table element",
			(char *) 0);

	TRACE2(tet_Tbuf, 6, "allocate systab element = %s", tet_i2x(sp));

	bzero((char *) sp, sizeof *sp);
	sp->sy_magic = SY_MAGIC;
	sp->sy_sysid = -1;
	sp->sy_activity = -1;

	return(sp);
}

#if 0	/* this function not used anywhere */
static void syfree(sp)
struct systab *sp;
{
	TRACE2(tet_Tbuf, 6, "free systab element = %s", tet_i2x(sp));

	if (sp) {
		ASSERT(sp->sy_magic == SY_MAGIC);
		bzero((char *) sp, sizeof *sp);
		free((char *) sp);
	}
}
#endif

/*
**	syadd() - add a systab element to the tcc system table
*/

static void syadd(sp)
struct systab *sp;
{
	tet_listinsert((struct llist **) &systab, (struct llist *) sp);
}

/*
**	syrm() - remove a systam element from the tcc system table
*/

#if 0	/* this function not used anywhere */
static void syrm(sp)
struct systab *sp;
{
	tet_listremove((struct llist **) &systab, (struct llist *) sp);
}
#endif

/*
**	syfind() - find the systab entry for the named system
**		and return a pointer thereto
**
**	return (struct systab *) 0 if not found
*/

struct systab *syfind(sysid)
int sysid;
{
	register struct systab *sp;

	for (sp = systab; sp; sp = sp->sy_next) {
		ASSERT(sp->sy_magic == SY_MAGIC);
		if (sp->sy_sysid == sysid)
			break;
	}

	return(sp);
}

/*
**	symax() - return the highest sysid in the systab
*/

int symax()
{
	register struct systab *sp;
	register int max = -1;

	for (sp = systab; sp; sp = sp->sy_next) {
		ASSERT(sp->sy_magic == SY_MAGIC);
		if (sp->sy_sysid > max)
			max = sp->sy_sysid;
	}

	return(max);
}

