/*
 *      SCCS:  @(#)tcmfuncs.c	1.27 (99/11/15) 
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1992 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 *
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
static char sccsid_tcmfuncs[] = "@(#)tcmfuncs.c	1.27 (99/11/15) TET3 release 3.6";
static char *copyright[] = {
	"(C) Copyright 1996 X/Open Company Limited",
	"All rights reserved"
};
#endif

/************************************************************************

SCCS:   	@(#)tcmfuncs.c	1.27 99/11/15 TETware release 3.6
NAME:		tcmfuncs.c
PRODUCT:	TETware
AUTHOR:		Andrew Dingwall, UniSoft Ltd.
DATE CREATED:	April 1992

DESCRIPTION:
	generic tcm client-related functions

MODIFICATIONS:
	Andrew Dingwall, UniSoft Ltd., October 1992
	Set tet_progname to basename(argv[0]) so as to avoid long path names
	in error messages.
	Moved tet_tcmptype() call to after TET_TIARGS parsing so that tcmchild
	processes can determine which system they are on.
	Moved tet_root[] from dtcm.c to here so that it is available
	to tcmchild processes as well.

	Denis McConalogue, UniSoft Limited, September 1993
	added ss_disconnect() function

	Andrew Dingwall, UniSoft Ltd., December 1993
	Removed disconnect stuff.
	Changed dapi.h to dtet2/tet_api.h
	Corrected error message when TET_ROOT is not set.

	Andrew Dingwall, UniSoft Ltd., February 1994
	replaced generror() with dtcmerror() as the error handler -
	this sends TETware errors to tet_error() so they will probably
	appear in the journal rather than just being sent to stderr
	straight off

	Andrew Dingwall, UniSoft Ltd., December 1994
	Handle recursive calls to dtcmerror() sensibly.

	Andrew Dingwall, UniSoft Ltd., August 1996
	changes for tetware tcc

	Geoff Clare, UniSoft Ltd., August 1996
	Missing <unistd.h>.

	Geoff Clare, UniSoft Ltd., Sept 1996
	Changes for TETware-Lite.

	Geoff Clare, UniSoft Ltd., Oct 1996
	Enable tracing in TETware-Lite.
	Restructured tcm source to avoid "ld -r".

	Andrew Dingwall, UniSoft Ltd., June 1997
	changes to enable parallel remote and distributed test cases
	to work correctly;
	improved process type determination when processing trace args

	Andrew Dingwall, UniSoft Ltd., July 1998
	Added support for shared API libraries.
 
	Andrew Dingwall, UniSoft Ltd., July 1999
	moved TCM code out of the API library

	Andrew Dingwall, UniSoft Ltd., October 1999
	added support for strict POSIX threads

************************************************************************/

/*
** This file is a component of the TCM (tcm.o) and/or one of the child
** process controllers (tcmchild.o and tcmrem.o).
** On UNIX systems, these .o files are built using ld -r.
** There is no equivalent to ld -r in MSVC, so on Win32 systems each .c
** file is #included in a scratch .c or .cpp file and a single object
** file built from that.
**
** This imposes some restictions on the contents of this file:
**
**	+ Since this file might be included in a C++ program, all
**	  functions must have both ANSI C and common C definitions.
**
**	+ The only .h file that may appear in this file is tcmhdrs.h;
**	  all other .h files that are needed must be #included in there.
**
**	+ The scope of static variables and functions encompasses all
**	  the source files, not just this file.
**	  So all static variables and functions must have unique names.
*/


/*
** all the header files are included by tcmhdrs.h
** don't include any other header files directly
*/
#include "tcmhdrs.h"


#ifdef NEEDsrcFile
#  ifdef srcFile
#    undef srcFile
#  endif
#  define srcFile srcFile_tcmfuncs
   static char srcFile[] = __FILE__;	/* file name for error reporting */
#endif




/*
**	tet_tcminit() - initialisation for both master and slave TCMs
*/

/* ARGSUSED */
#ifdef PROTOTYPES
void tet_tcminit(int argc, char **argv)
#else
void tet_tcminit(argc, argv)
int argc;
char **argv;
#endif
{
	register char *envstring, *p;
	char **args;
	int nargs;
	static char tiargs_name[] = "TET_TIARGS";
	static char tetroot_name[] = "TET_ROOT";
	static char envmsg[] = "null or not set";
#ifdef NOTRACE
	int twarn = 0;
#else
#endif

	/* get TET_ROOT out of the environment */
	if ((envstring = getenv(tetroot_name)) == (char *) 0 || !*envstring)
		fatal(0, tetroot_name, envmsg);
	(void) sprintf(tet_root, "%.*s", (int) sizeof tet_root - 1, envstring);

	/* get the dtet ti args out of the environment and count them */
	if ((envstring = getenv(tiargs_name)) == (char *) 0 || !*envstring) {
		args = NULL;
		nargs = 0;
	}
	else {
		nargs = 1;
		for (p = envstring; *p; p++)
			if (isspace(*p))
				nargs++;

		/* allocate some space for argument pointers */
		errno = 0;
		if ((args = (char **) malloc(nargs * sizeof *args)) == (char **) 0)
			fatal(errno, "can't get memory for arg list",
				(char *) 0);
		TRACE2(tet_Tbuf, 6, "allocate ti env args = %s", tet_i2x(args));

		/* split the arg string into fields */
		nargs = tet_getargs(envstring, args, nargs);


	} /* end of tiargs processing */


#ifndef NOTRACE
	/* initialise tracing for known process type */
	if (args != NULL) {
		tet_tfclear();
		tet_traceinit(nargs + 1, args - 1);
	}
#endif

	if (args != NULL) {
		TRACE2(tet_Tbuf, 6, "free ti env args = %s", tet_i2x(args));
		free((char *) args);
	}


}

/*
**	tet_dtcmerror() - TETware TCM error printing routine
**
**	messages printed by the error() macro come here
**	note that this function might be called recursively
**
**	This function must not call any non async-signal safe functions
**	when called from the child of a multi-threaded parent in strict
**	POSIX threads mode.
*/

#ifdef PROTOTYPES
void tet_dtcmerror(int errnum, char *file, int line, char *s1, char *s2)
#else
void tet_dtcmerror(errnum, file, line, s1, s2)
int errnum, line;
char *file, *s1, *s2;
#endif
{
	char msg[MAXPATH + 128];
	char *p1, *p2;
	int combined_save = 0;
	static int inprogress = 0;

	/* start the buffer with filename and line number */
	p1 = msg;
	*p1++ = '(';
	p2 = tet_basename(file);
	while (*p2 && p1 < &msg[sizeof msg - 3])
		*p1++ = *p2++;
	*p1++ = ',';
	*p1++ = ' ';
	p2 = tet_i2a(line);
	while (*p2 && p1 < &msg[sizeof msg - 3])
		*p1++ = *p2++;
	*p1++ = ')';
	*p1++ = ' ';

	/* append the first message string */
	while (*s1 && p1 < &msg[sizeof msg - 1])
		*p1++ = *s1++;

	/* append the second message string if there is one */
	if (s2 && *s2 && p1 < &msg[sizeof msg - 1]) {
		*p1++ = ' ';
		while (*s2 && p1 < &msg[sizeof msg - 1])
			*p1++ = *s2++;
	}

	/*
	** terminate the message and punt it to tet_error() for output -
	**	if we are called recursively, there is probably
	**	something wrong with the connection to xresd, so clear
	**	the tet_combined_ok flag to force output to stderr instead
	*/
	*p1 = '\0';
	if (inprogress++) {
		combined_save = tet_combined_ok;
		tet_combined_ok = 0;
	}
	tet_error(errnum, msg);
	if (--inprogress)
		tet_combined_ok = combined_save;

	errno = 0;
}



